using System;
using System.Collections.Generic;
using System.Text;
using System.DirectoryServices;
using System.Collections;
using System.Xml;
using System.IO;

namespace ADHelper
{
    public class User
    {
        /*
         * Get all users and the groups that they are a member of; write out to tab delimited file
         * 
         * Example: GetUserGroups("VS", "LDAP://dc=vs,dc=local")
         */
        public void GetUserGroups(string domain, string path, string filepath)
        {
            // Step 1
            DirectoryEntry searchRoot = new DirectoryEntry(path, null, null, 
                AuthenticationTypes.Secure);
            DirectorySearcher dirSearch = new DirectorySearcher(searchRoot);
            dirSearch.SearchScope = SearchScope.Subtree;
            dirSearch.PropertiesToLoad.Add("samaccountname");
            dirSearch.PropertiesToLoad.Add("memberof");
            dirSearch.Filter = "(objectclass=person)";
            dirSearch.PageSize = 1000;

            string samaccountname = String.Empty;
            ArrayList groups;
            int personcount = 0;
            int objectcount = 0;

            using (searchRoot) {    // Step 2
              using (SearchResultCollection results = dirSearch.FindAll()) {
                objectcount = results.Count;
                using (StreamWriter sw = new StreamWriter(filepath)) { // Step 3
                  foreach (SearchResult result in results) { // Step 4
                    ++personcount;
                    groups = new ArrayList();
                    // Step 5
                    ResultPropertyCollection props = result.Properties;
                    foreach (DictionaryEntry entry in props) {
                      ResultPropertyValueCollection values = entry.Value 
                          as ResultPropertyValueCollection;
                      if (entry.Key.ToString() == "samaccountname")
                          samaccountname = GetSingleValue(values);
                      if (entry.Key.ToString() == "memberof")
                          groups = GetGroups(values);
                    }
                    foreach (string group in groups) { // Step 6
                      sw.WriteLine("{0}\t{1}\t{2}", samaccountname, domain, 
                          group);
                    }
                 }
                 sw.Flush();
              }
            }
          }
        }

        ArrayList GetGroups(ResultPropertyValueCollection values)
        {
            ArrayList valueList = new ArrayList();

            foreach (object val in values)
            {
                string memberof = val.ToString();
                string[] pairs = memberof.Split(',');
                string[] group = pairs[0].Split('=');
                valueList.Add(group[1]);
            }

            return valueList;
        }

        string GetSingleValue(ResultPropertyValueCollection values)
        {
            foreach (object val in values)
            {
                return val.ToString();
            }

            return null;
        }
    }
}
